import os
from collections import namedtuple
from subprocess import check_output
from .config import DATA_DIR

NMAP_SERVICES_PATH = os.path.join(DATA_DIR, 'nmap-services.txt')
PORTMAP = {}


PortMapping = namedtuple('PortMapping', 'name port proto')
NetstatLine = namedtuple('NetstatLine', 'proto recvq sendq local_ip local_port '
                         'foreign_ip foreign_port state pid prog local_name '
                         'foreign_name')


def _parse_portmap_line(line):
    line = line.strip()
    if not line or line.startswith('#'):
        return None
    name, port_proto = line.split()[:2]
    port, proto = port_proto.split('/')
    return PortMapping(
        name=name,
        port=int(port),
        proto=proto,
    )


def _convert_port(p):
    if p.isdigit():
        return int(p)
    else:
        return p


def load_portmap():
    if PORTMAP:
        return
    with open(NMAP_SERVICES_PATH) as f:
        for line in f:
            pm = _parse_portmap_line(line)
            if pm is None:
                continue
            PORTMAP[pm.proto, pm.port] = pm.name


def netstat(debug=False):
    load_portmap()
    out = check_output(['netstat', '--ip', '-pltn']).decode('utf8')
    netstats = []
    for line in out.splitlines():
        spl = line.split()
        if debug:
            print('parsing {!r}'.format(spl))
        proto = spl[0]
        if proto not in ['tcp', 'udp']:
            if debug:
                print('skipping')
            continue
        recvq = spl[1]
        sendq = spl[2]
        local_ip, local_port = spl[3].split(':')
        local_port = _convert_port(local_port)
        local_name = None
        if isinstance(local_port, int):
            local_name = PORTMAP.get((proto, local_port))
        foreign_ip, foreign_port = spl[4].split(':')
        foreign_port = _convert_port(foreign_port)
        foreign_name = None
        if isinstance(foreign_port, int):
            foreign_name = PORTMAP.get((proto, foreign_port))
        state = spl[5]
        pid_prog = spl[6]
        if pid_prog and pid_prog != '-':
            pid, prog = pid_prog.split('/', 1)
        else:
            pid, prog = None, None
        nsl = NetstatLine(
            proto=proto,
            recvq=int(recvq),
            sendq=int(sendq),
            local_ip=local_ip,
            local_port=local_port,
            foreign_ip=foreign_ip,
            foreign_port=foreign_port,
            state=state,
            pid=int(pid) if pid is not None else None,
            prog=prog,
            local_name=local_name,
            foreign_name=foreign_name,
        )
        netstats.append(nsl)
    return netstats
